package com.yihu.hos.standard.service.standard;


import com.yihu.hos.standard.model.standard.STDVersion;
import com.yihu.hos.standard.util.CDAVersionUtil;
import com.yihu.hos.web.framework.baseservice.BaseService;
import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 版本管理实现类。
 * @author lincl
 * @version 1.0
 * @created 2016.2.3
 */
@Service
public class STDVersionService extends BaseService<STDVersion> {
    public static final String FBVersion = "000000000000";

//    @Autowired
//    StdSessionFactoryBean sessionFactory;

    /**
     * 创建一个阶段性版本. 参数中的版本不能处于编辑状态.
     *
     * @param baseVersion
     * @param author
     */
    @Transactional(propagation = Propagation.REQUIRED)
    public STDVersion createStageVersion(STDVersion baseVersion, String author) throws Exception {
        if (author == null || author.length() == 0) throw new IllegalArgumentException("作者不能为空");

        STDVersion stagedVersion = null;
        if (baseVersion == null) {
            if (getVersionList().length != 0) {
                throw new IllegalArgumentException("基础版本不能为空");
            } else {
                // 空库的情况下，使用 XCDAVersionManager.FBVersion 作为初始版本
                stagedVersion = new STDVersion(null, author, "0.1");
                stagedVersion.setVersionName("0.1");
                stagedVersion.setVersion(STDVersionService.FBVersion);
            }

        } else {
            stagedVersion = new STDVersion(baseVersion.getVersion(), author, baseVersion.getVersionName());
        }

        buildVersionTables(stagedVersion);
        saveEntity(stagedVersion);
//        createVersionedClass(stagedVersion);
        return stagedVersion;
    }

//    private void createVersionedClass(STDVersion stagedVersion) throws Exception {
//        String version = stagedVersion.getVersion();
//        List<Class> tableClass = new ArrayList<>();
//        for (String vesionedEntity : BeanConfig.vesionedEntitys.keySet()){
//            tableClass.add(
//                    ClassPoolUtils.tableMapping(
//                            vesionedEntity,
//                            BeanConfig.vesionedEntitys.get(vesionedEntity) + version,
//                            vesionedEntity + version));
//        }
//        sessionFactory.addClassToBuildSessionFactory(tableClass.toArray(new Class[tableClass.size()]));
//    }

    /**
     * 根据版本号获取版本信息
     *
     * @param version
     * @return
     */
    public STDVersion getVersion(String version) throws ParseException {
        List<STDVersion> list = search("version=" + version);
        if(list!=null && list.size()>0){
            return list.get(0);
        }
        return null;
    }

    /**
     * 获取最新的已发布版本
     *
     * @return
     */
    public STDVersion getLatestVersion() {
        Session session = currentSession();

        Query query = session.createQuery("from STDVersion a where a.inStage = false order by a.version desc");
        query.setFirstResult(0);
        query.setMaxResults(1);

        List<STDVersion> versions = query.list();
        STDVersion cdaVersion = versions.size() > 0 ? versions.get(0) : null;

        return cdaVersion;
    }

    /**
     * 获取所有版本
     *
     * @return
     */
    public STDVersion[] getVersionList() {
        Session session = currentSession();
        Query query = session.createQuery("from STDVersion order by commitTime");
        List<STDVersion> versionList = query.list();
        return versionList.toArray(new STDVersion[versionList.size()]);
    }

    /**
     * 发布版本
     *
     * @param version
     */
    @Transactional(propagation = Propagation.REQUIRED)
    public void commitVersion(STDVersion version) throws Exception {

        version.setInStage(false);
        version.setCommitTime(new Date());
        saveEntity(version);
    }


    @Transactional(propagation = Propagation.REQUIRED)
    public void revertVersion(STDVersion version) throws Exception {

        dropVersionTables(version);
        deleteEntity(version);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void dropVersion(STDVersion version) throws Exception {
        Session session = currentSession();
        Query query = session.createQuery("from STDVersion where baseVersion = :version");
        query.setString("version", version.getVersion());
        STDVersion subVersion = (STDVersion) query.uniqueResult();
        if (subVersion != null) {
            subVersion.setBaseVersion(version.getBaseVersion());
            session.save(subVersion);
        }
        dropVersionTables(version);
        deleteEntity(version);
    }

    /**
     * 回滚到编辑状态
     *
     * @param version
     */
    @Transactional(propagation = Propagation.REQUIRED)
    public void rollbackToStage(STDVersion version) throws Exception {
        // 已经是编辑状态，直接返回
        if (version.isInStage()) {
            return;
        }

        // 非最新版本，无法回滚
        STDVersion latestVersion = getLatestVersion();
        if (latestVersion != null && !latestVersion.getVersion().equals(version.getVersion())) {
            throw new IllegalArgumentException("非最新版本,无法回滚");
        }

        // 回滚
        version.setInStage(true);
        version.setCommitTime(new Date());
        saveEntity(version);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    private void dropVersionTables(STDVersion stdVersionVersion) {
        String version = stdVersionVersion.getVersion();
        String datasetTable = CDAVersionUtil.getDataSetTableName(version);
        String metadataTable = CDAVersionUtil.getMetaDataTableName(version);
        String dictTable = CDAVersionUtil.getDictTableName(version);
        String dictEntryTable = CDAVersionUtil.getDictEntryTableName(version);
        String cdaDocumentTable = CDAVersionUtil.getCDATableName(version);
        String relationshipTable = CDAVersionUtil.getCDADatasetRelationshipTableName(version);

        Session session = currentSession();

        String[] toDropTables = {datasetTable, metadataTable, dictTable, dictEntryTable, cdaDocumentTable, relationshipTable};
        for (String table : toDropTables) {
            Query query = session.createSQLQuery("DROP TABLE IF EXISTS " + table);
            query.executeUpdate();
        }
    }

    @Transactional(propagation = Propagation.REQUIRED)
    private void buildVersionTables(STDVersion cdaVersion) {
        //为空表示这是一个初始版本，不需要再创建表
        if (cdaVersion.getBaseVersion() == null) return;
        String version = cdaVersion.getVersion();
        String[] newTables = {CDAVersionUtil.getDataSetTableName(version),
                CDAVersionUtil.getMetaDataTableName(version),
                CDAVersionUtil.getDictTableName(version),
                CDAVersionUtil.getDictEntryTableName(version),
                CDAVersionUtil.getCDATableName(version),
                CDAVersionUtil.getCDADatasetRelationshipTableName(version)};

        version = cdaVersion.getBaseVersion();
        String[] baseTables = {CDAVersionUtil.getDataSetTableName(version),
                CDAVersionUtil.getMetaDataTableName(version),
                CDAVersionUtil.getDictTableName(version),
                CDAVersionUtil.getDictEntryTableName(version),
                CDAVersionUtil.getCDATableName(version),
                CDAVersionUtil.getCDADatasetRelationshipTableName(version)};

        Session session = currentSession();
        for (int i = 0; i < baseTables.length; ++i) {
            String baseTable = baseTables[i];
            String newTable = newTables[i];

            Query query = session.createSQLQuery("CREATE TABLE " + newTable + " like " + baseTable);
            query.executeUpdate();

            query = session.createSQLQuery("INSERT INTO " + newTable + " SELECT * FROM " + baseTable);
            query.executeUpdate();
        }
    }

    //1多条件查询
    public List<STDVersion> searchVersions(Map<String, Object> args) {
        Session session = currentSession();
        String version = (String) args.get("version");
        String versionName = (String) args.get("versionName");
        Integer page = (Integer) args.get("page");
        Integer pageSize = (Integer) args.get("rows");
        String hql = "from STDVersion where (version like :version or versionName like :versionName) order by commitTime";
        Query query = session.createQuery(hql);
        query.setString("version", "%" + version + "%");
        query.setString("versionName", "%" + versionName + "%");
        if (page != 0) {
            query.setMaxResults(pageSize);
            query.setFirstResult((page - 1) * pageSize);
        }
        return query.list();
    }

    //2查询符合条件记录数
    public Integer searchVersionInt(Map<String, Object> args) {
        Session session = currentSession();
        String version = (String) args.get("version");
        String versionName = (String) args.get("versionName");
        String hql = "SELECT count(*) FROM STDVersion WHERE (version like :version or versionName like :versionName)";
        Query query = session.createQuery(hql);
        query.setString("version", "%" + version + "%");
        query.setString("versionName", "%" + versionName + "%");
        return Integer.parseInt(query.list().get(0).toString());
    }

    /**
     * 根据标准ID，获取标准版本集合
     * @param standardId 标准ID
     * @return 标准版本集合
     */
    public List<STDVersion> findVersionsByStdId(Long standardId) {
        Criteria criteria = currentSession().createCriteria(STDVersion.class);
        criteria.add(Restrictions.eq("standardId", standardId));
        return criteria.list();
    }

    public Integer checkVersionName(String versionName) {
        Session session = currentSession();
        String hql = "SELECT count(*) FROM STDVersion WHERE versionName =:versionName";
        Query query = session.createQuery(hql);
        query.setString("versionName", versionName);
        return Integer.parseInt(query.list().get(0).toString());

    }

    public Integer searchInStage() {
        Session session = currentSession();
        Query query = session.createQuery("SELECT count(*) FROM STDVersion WHERE inStage = true");
        return Integer.parseInt(query.list().get(0).toString());
    }
}